/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wBlowing.h"
#include "wTag.h"
#include "wElement.h"
#include "wNode.h"
using namespace tbox;
using namespace dart;

Blowing::Blowing(std::shared_ptr<MshData> _msh, int no) : Group(_msh, no)
{
    create();
}

Blowing::Blowing(std::shared_ptr<MshData> _msh, std::string const &name) : Group(_msh, name)
{
    create();
}
Blowing::~Blowing()
{
    for (auto p : uE)
        delete p.second;
    std::cout << "~Blowing()\n";
}

/**
 * @brief Create data structure
 */
void Blowing::create()
{
    // Get the nodes of Blowing
    for (auto e : tag->elems)
        for (auto n : e->nodes)
            nodes.push_back(n);
    std::sort(nodes.begin(), nodes.end());
    auto iterator = std::unique(nodes.begin(), nodes.end());
    nodes.resize(std::distance(nodes.begin(), iterator));

    // Initialize blowing velocity
    uE.resize(tag->elems.size());
    for (size_t i = 0; i < tag->elems.size(); ++i)
    {
        F0ElC *f = new F0ElC(0.);
        uE[i] = std::pair<Element *, F0ElC *>(tag->elems[i], f);
    }
}

/**
 * @brief Set blowing velocity
 */
void Blowing::setU(size_t i, double ue)
{
    uE[i].second->update(ue);
}

double Blowing::getU(size_t i) const
{
    // Workaround to return a constant value
    tbox::Element *e = uE[i].first;
    std::vector<double> phi(0, 0.);
    return uE[i].second->eval(*e, phi, 0);
}

void Blowing::write(std::ostream &out) const
{
    out << " Blowing boundary condition on " << *tag << std::endl;
}
